/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 * C Example Program : CDMA2000 Signalling Measurements
 *
 * The following example explains how to handle primary and secondary
 * addressing and how to perform a simple measurement with the CMU. In the
 * example, remote control via GPIB bus and the programming language C is used.
 *
 * The example requires CMU options :
 *	
 *	CMU200
 *  CMU-B83	- CDMA2000 LH
 *	CMU-K84 - SW package for CDMA2000 US cellular band
 *  CMU-K85 - SW package for CDMA2000 PCS 1900 band
 *  
 *	FW 3.05 or greater
 *
 *  rscmu200 CVI driver - Rev 3.10.04
 *  rscmuc2k CVI driver - Rev 3.11RC02 
 *
 * Compiler name and release(s) : LabWindows/CVI 5.5
 * Original Release : January, 2002
 *
 *
 * Version : 1.1
 *
 * (C)opyright 2002-03 ROHDE & SCHWARZ All Rights Reserved.
 *
 * ----------------------------------------------------------------------------*/
 
#include "ansi_c.h"
#include "userint.h"
#include "utility.h"
#include "rscmu200.h"
#include "rscmuc2k.h"

#define CDMA 3

// Structure handle
typedef struct {

	ViSession	BASE;
	ViSession	RF;
	ViSession	AUDIO;
	ViSession	GSM400S;
	ViSession	GSM400NS;
	ViSession	GSM850S;
	ViSession	GSM850NS;
	ViSession	GSM900S;
	ViSession	GSM900NS;
	ViSession	GSM1800S;
	ViSession	GSM1800NS;
	ViSession	GSM1900S;
	ViSession	GSM1900NS;
	ViSession	CDMA800S;
	ViSession	CDMA800NS;	
	ViSession	CDMA1900S;
	ViSession	CDMA1900NS;
	ViSession	AMPSS;
	ViSession	AMPSNS;
	ViSession	VALID;

} tHandle;

// Base station structure
typedef struct {
    ViInt32			Netw;
    ViInt32			RFChan;
    ViReal64		PowerLev;
    ViInt32			TrafficChan;
    ViReal64		TrafficLev;
    ViReal64		PagingLev;
    ViReal64		PilotLev;
    ViReal64		SyncLev;
    ViInt32			PNOffset;
    ViInt32			FrameOffset;
    ViInt32			RATE;
    ViInt32			CallMode;
    ViInt32			NETW_SID;
    ViInt32			NETW_NID;
	ViInt32			NET_PREV;        
    ViReal64		regperiod;
    ViInt32			ServClass;
    ViInt32			ServOption;
    ViInt32			RC_FFCH;
    ViInt32			RC_RFCH;
    ViInt32			CCH;
    ViReal64		CCH_RFLev;
    ViInt32			VCH;
    ViReal64		VCH_RFLev;    
    ViInt32			CMAC;
    ViInt32			VMAC;
	
} tBST;

// Measurement structure
// TX NPOWer Object
typedef struct {

	ViReal64 AvgCur;
	ViReal64 MinCur; 
	ViReal64 MaxCur;
	ViReal64 AvgAvg; 
	ViReal64 MinAvg; 
	ViReal64 MaxAvg; 

} tNPOW;

// TX Min-Max power structure
typedef struct {

   ViReal64    	PowCurr;
   ViReal64    	PowAvg;
   ViReal64    	PowMin;
   ViReal64    	PowMax;
   ViReal64    	RhoCurr;
   ViReal64    	RhoAvg;
   ViReal64    	RhoMin;
   ViReal64    	RhoMax;
   ViReal64    	MOT;  
   
} tMMPOW;

// TX Modulation overview measurement structure
typedef struct  {
	ViReal64	PowExpected;
	ViReal64	PowMeasured;
	ViReal64	FreqExpected;
	ViReal64	FreqMeasured;
	ViReal64	TimErr;
	ViReal64	Rho;
	
} tOVERVIEW;

// RX Receiver measurement structure
typedef struct {
	ViReal64 	PilotActual;
	ViReal64 	PilotPower;	
	ViReal64	FER;
	ViInt32		Errors;
	ViInt32		FramesReceived;
	ViReal64	CL;
} tRX;

typedef struct {
	ViReal64	Power;
	ViInt32		LimMATC;
	ViReal64	MOT;
} tGATED;

// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif


// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
	ViChar l_vCmuErrMsg[1024];
	ViChar l_vErrBuf[1024];
	ViInt32 _error;
    
	if (CmuStatus != VI_SUCCESS)
	{

		//RSCMU_Base_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
		RSCMU_CDMA_errorMessage(VI_NULL, CmuStatus, l_vCmuErrMsg);

		//Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
		sprintf (l_vErrBuf, "Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);        
		MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);
	
	return -1;
	}
    return 0;
    
Error:
    return -1;

}

int RF_Path(ViSession	handle, int	Netw, int RFIO, double RFATT);
int	Setup_CDMA_BST( ViSession	handle, tBST	BST);
int Setup_CDMA_Service(ViSession 	handle, tBST	BST);
int Setup_CDMA_Measurement( ViSession	handle, tBST	BST);



/************************************************************************************************/
//
/************************************************************************************************/
int main(void) 
{

	// Init variable
	tHandle				handle 			= {NULL};
	tBST				BST 			= {NULL};
	tNPOW				TX_NPOW 		= {NULL};
	tMMPOW				TX_MMPOW		= {NULL};
	tGATED				TX_GATED		= {NULL};
	CODE_DOMAIN_POWER	TX_CDP			= {NULL};
	OVERVIEW_C2K		TX_OVERMEAS 	= {NULL};
	OVERVIEW			TX_MOD_OVER		= {NULL};
	ERROR_MOD			TX_MOD_EVM		= {NULL};
	tRX					RX_FER			= {NULL};
	
	ViChar				e[1024]			= "\0";
	ViChar				OPT[512]		= "\0";
	ViChar				buffout[512] 	= "\0";
	ViChar				buffin[512] 	= "\0";
	ViChar				MATCH[4][80]		= {"Tolerance value matched\0", "Measurement invalid\0","Tolerance value exceeded\0","Tolerance value underflow\0"};

	ViStatus			cmu_status		= 0;		
	int					i				= 0;
	int					_error			= 0;	
	int 				index 			= 0;
	int					TestLoop		= 0;
	int					DispStatus		= 0;
	ViInt32				state 			= -1;
	ViInt32				meas_status		= -1;
	ViInt32				counterStat		= -1; 
	ViInt32				counterEval		= -1;
	ViInt32				MeasMod			= RSCMU_OQPSK;
	
	
	/************************************************************************************************/
	//
	/************************************************************************************************/
	
	Cls();
	printf("\nR&S CMU200 - CDMA2000 call processing example\n");
	
	// Init the base driver (open handle with the rscmu200.dll)
	printf("\tGet handle with the base driver rscmu200.dll\n");
	CheckCmuError(RSCMU_Init_Base (&handle.BASE, RSCMU_GPIB, 0, 20,RSCMU_DO_QUERY, RSCMU_RESET_ON, e));

	// Open trace file and GPIB exchange
	printf("\tStore the GPIB exchange in a log file\n");
	CheckCmuError(RSCMU_Trace_ON (handle.BASE, "RsCmu_C2kSIG.iee", e));
	 
	// Retreive CMU options
	printf("\tRetreive the CMU options\n");
	CheckCmuError(RSCMU_Query (handle.BASE, "*OPT?", OPT, RSCMU_OFF, e));
	 
	// Test if CMU supports CD2K
	if (strstr(OPT,"B83") == NULL ) 
	{
		strcpy(e,"Option B83 required for this example\0");
		goto Error;
	};
	
	// Test if CMU supports K84 SW package
	if (strstr(OPT,"K84") == NULL ) 
	{
		strcpy(e,"Example required K84 SW package CDMA2K US cell band\0");
		goto Error;
	};

	// Test if CMU supports K85 SW package
	if (strstr(OPT,"K85") == NULL ) 
	{
		strcpy(e,"Example required K85 SW package CDMA2K PCS band\0");
		goto Error;
	};
	
	/************************************************************************************************/
	// Map the GPIB secondary addresses with the CDMA200 K84 and K85 functionnal group
	/************************************************************************************************/

	// Map the secondary addresses
	// Map the US cellular Non Signalling Group
	printf("\tAssign secondary adresses\n");
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 2, RSCMU_CDMA2KCELLMS_NSIG, e))
	// Map the US cellular Signalling Group
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 3, RSCMU_CDMA2KCELLMS_SIG, e));
	// Map the PCS cellular Non Signalling Group
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 4, RSCMU_CDMA2KPCSMS_NSIG, e));
	// Map the PCS cellular Signalling Group
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 5, RSCMU_CDMA2KPCSMS_SIG, e));

	/************************************************************************************************/
	// Get handles
	/************************************************************************************************/
	printf("\tGet handle for CDMA800 sig group & CDMA1900 Sig group (open handles with the rscmuc2k.dll)\n");
	CheckCmuError(RSCMU_Init_CDMA2K (handle.BASE, &handle.CDMA800S,RSCMU_CDMA2KCELLMS_SIG, e));
	CheckCmuError(RSCMU_Init_CDMA2K (handle.BASE, &handle.CDMA1900S,RSCMU_CDMA2KPCSMS_SIG, e));
	
	// Set Timeout
	printf("\tSet Timeout\n");
	CheckCmuError(RSCMU_SetTimeoutAttribute (handle.CDMA800S, 7000, e));
	CheckCmuError(RSCMU_SetTimeoutAttribute (handle.CDMA1900S, 7000, e));

	/************************************************************************************************/
	// Configure CDMA network and base station
	/************************************************************************************************/
	printf("\tTurn OFF signaling\n");
	// !!! If you use FW V3.05 start signaling mode 
	RSCMU_Query (handle.CDMA800S, "PROC:SIGN:ACT SON", buffin, RSCMU_ON, e);
	// Stop signaling in US cellular band
	CheckCmuError(RSCMU_CDMA2K_ProcSigAction (handle.CDMA800S, RSCMU_SIG_ACT_SOFF, e));
	//
	// Stop signaling in PCS band
	CheckCmuError(RSCMU_CDMA2K_ProcSigAction (handle.CDMA1900S, RSCMU_SIG_ACT_SOFF, e));

	/************************************************************************************************/
	// Set RF Path
	/************************************************************************************************/
	// Select the RF input and Output and Set the RF cable compensation
	printf("\nSet RF path and RF attenuation\n");
	if (RF_Path(handle.CDMA800S, CDMA, RSCMU_INPUT_RF2, 0.7) != 0) goto Error;
	if (RF_Path(handle.CDMA1900S, CDMA, RSCMU_INPUT_RF2, 1.5) != 0) goto Error;		

	/************************************************************************************************/
	// Select band
	/************************************************************************************************/
 	// set handle
 	handle.VALID =  handle.CDMA800S;
	
	/************************************************************************************************/
	// Set BASE STATION RF and NETWORK parameters
	/************************************************************************************************/
	// Setup BST for US Cell band and IS95 call
	printf("\nConfigure CDMA Base station & Network in US Cellular Band\n");

    BST.Netw 		= RSCMU_C2K_USC;							// Band
    BST.RFChan  	= 283;										// Base Station Primary Channel - Mandatory
    BST.PowerLev	= -50.0;									// Base Station RF ouput power  in dBm
    BST.TrafficChan = 8;										// Forward Traffic Channel
    BST.TrafficLev	= -14.0;									// Traffic Channel level in dB
    BST.PagingLev	= -12.0;									// Paging Channel Level in dB
    BST.PilotLev	= -7.0;										// Pilot Channel level in dB
    BST.SyncLev		= -16.0;									// Synchro Channel Level in dB
    BST.PNOffset	= 0;										// PN Offset
    BST.FrameOffset = 0;										// Frame Offset
    BST.NETW_NID	= 1;										// Network ID
    BST.NETW_SID	= 2;										// Base Station System IDentification SID - Mandatory
    BST.NET_PREV	= 6;										// CDMA Protocol revision - Should match with the phone
	
	if (Setup_CDMA_BST(handle.VALID, BST) <0) goto Error;

	
 

// Program loop starts here
TestLoop = 1;
do
{

	// Display Message and Query item
	DispStatus = GenericMessagePopup ("Message Operator","For a new test Press <OK>\nFor exit <Cancel>","OK", "Cancel", "", 
			0, 0, 0,
			VAL_GENERIC_POPUP_BTN1,
			VAL_GENERIC_POPUP_BTN1,
			VAL_GENERIC_POPUP_BTN2);
			
	if (DispStatus == 2) {
		TestLoop = 0;
		break;
	};

	
	// test if phone support CDMA2000 or only CDMA one
	if (BST.NET_PREV < 6)
	
	
		// Display Message and Query item
		DispStatus = GenericMessagePopup ("Message Operator","Set up a call\nRC1 - 9.6 Kbit/s\nRC2 - 14.4 Kbit/s","RC1", "RC2", "", 
				0, 0, 0,
				VAL_GENERIC_POPUP_BTN1,
				VAL_GENERIC_POPUP_BTN1,
				VAL_GENERIC_POPUP_BTN2);
	else
	
		// Display Message and Query item
		DispStatus = GenericMessagePopup ("Message Operator","Set up a call\nRC1 - 9.6 Kbit/s\nRC2 - 14.4 Kbit/s\nRC4 - CDMA2000","RC1", "RC2", "RC4", 
				0, 0, 0,
				VAL_GENERIC_POPUP_BTN1,
				VAL_GENERIC_POPUP_BTN1,
				VAL_GENERIC_POPUP_BTN2);
			
	printf("\tConfigure the service configuration\n");
	if (DispStatus == 1) 
	{
	    BST.RATE		= RSCMU_CDMA_FULL;							// Set RATE mode to FULL
	    BST.ServClass 	= RSCMU_SERVICE_LOOP;						// Set Service to Test loopback
	    BST.ServOption  = RSCMU_SERV_OPT2_S02D;						// Set Service Option to 2 => 9.6 Kbit/s
	    BST.RC_FFCH		= 1;										// Set Forward Radio configuration to 1
	    BST.RC_RFCH		= 1;										// Set Reverse Radio configuration to 1
	
	}
	
	if (DispStatus == 2) 
	{
	    BST.RATE		= RSCMU_CDMA_FULL;							// Set RATE mode to FULL
	    BST.ServClass 	= RSCMU_SERVICE_LOOP;						// Set Service to Test loopback
	    BST.ServOption  = RSCMU_SERV_OPT2_S09D;						// Set Service Option to 9 => 14.4 Kbit/s
	    BST.RC_FFCH		= 2;										// Set Forward Radio configuration to 2
	    BST.RC_RFCH		= 2;										// Set Reverse Radio configuration to 2
	
	}
	
	if (DispStatus == 3) 
	{
	    BST.RATE		= RSCMU_CDMA_FULL;							// Set RATE mode to FULL
	    BST.ServClass 	= RSCMU_SERVICE_LOOP;						// Set Service to Test loopback
	    BST.ServOption  = RSCMU_SERV_OPT2_S09D;						// Set Service Option to 9 
	    BST.RC_FFCH		= 5;										// Set Forward Radio configuration to 5
	    BST.RC_RFCH		= 4;										// Set Reverse Radio configuration to 4
	
	}

	// Setup the service class and service configuration
	printf("\ntest phone in loopback\nSetup Radio Configuration Service to RC%1d\n",BST.RC_RFCH);

	
	if (Setup_CDMA_Service(handle.VALID, BST) <0) goto Error;

	if ((BST.RC_RFCH == 1) || (BST.RC_RFCH == 2)) MeasMod = RSCMU_OQPSK;
	if ((BST.RC_RFCH == 3) || (BST.RC_RFCH == 4)) MeasMod = RSCMU_HPSK;
 	
 	// Start signaling mode in US Cellular band
 	printf("\n\tStart signaling mode in US Cellular band\n");
 	CheckCmuError(RSCMU_CDMA2K_ProcSigState (handle.VALID, &state, e));
 	
 	// Start signaling
 	if (state == RSCMU_SIG_ACT_SOFF) CheckCmuError(RSCMU_CDMA2K_ProcSigAction (handle.VALID, RSCMU_SIG_ACT_SON, e));	
	
	
	// Set the RF primary channel
	printf("\n\tSet the RF primary channel\n");
	CheckCmuError(RSCMU_CDMA2K_ConfBSSRfChannel (handle.VALID, BST.RFChan, e));
	
	// Set CMU Ouput RF Power
	printf("\n\tSet CMU Ouput RF Power\n"); 
	BST.PowerLev = -50.0;
	CheckCmuError(RSCMU_CDMA2K_ConfBSSRfPowCDMA (handle.VALID, BST.PowerLev, e));
	

	DispStatus = MessagePopup ("MessageOperator", "(1)Switch On MS\n(2)Wait for service\n(3)Press Enter to continue");
	
	
	/************************************************************************************************/
	// Registration
	/************************************************************************************************/
	printf("\n\tRegister a phone \n");
	state = 0;
	index = 0;
	do {
		
	 	Delay(0.6);
	 	RSCMU_CDMA2K_ProcSigState (handle.VALID, &state, e);
		printf("!");
		
		if (index == 100) 
		{
			strcpy(e,"\nregistration failed!!!\0");
			TestLoop = 0;
			continue;
		}
	
	} while ((index <= 100 ) && (state != RSCMU_CDMA_STATE_REG));


	Delay(2.0);
	
	/************************************************************************************************/
	// Setup a call with MS
	/************************************************************************************************/
	printf("\n\n\tSetup a test loopback RC%1d call\n",BST.RC_RFCH);
	
	index = 0;
	do {
		
		// Page MS
		cmu_status = RSCMU_CDMA2K_PagingCallToMS (handle.VALID, 30, e);
		
		// Query CMU state
		CheckCmuError(RSCMU_CDMA2K_ProcSigState (handle.VALID, &state, e));
		if (state != RSCMU_CDMA_STATE_CEST) 
			Delay(3.0);
			
		else
			printf("\nMS in a call\n");
	
		index++;
		
	} while (index < 2);
	
	printf("\nTEST MS IN A RC%1d CALL\n",BST.RC_RFCH);
	
	// Query CMU state and test if MS is in a call with CMU
	CheckCmuError(RSCMU_CDMA2K_ProcSigState (handle.VALID, &state, e));
	if (state != RSCMU_CDMA_STATE_CEST) goto Error;

	/************************************************************************************************/
	// TEST MS in a call
	/************************************************************************************************/

	Delay(4.0);
	
	// Change RF channel during a call
	printf("\n\tChange channel during a call\n");
	CheckCmuError(RSCMU_CDMA2K_ProcBSSRfChannel (handle.VALID, 384, e));
	
	// Test if MS is still in a call after channel change
	CheckCmuError(RSCMU_CDMA2K_ProcSigState (handle.VALID, &state, e));
	if (state != RSCMU_CDMA_STATE_CEST) goto Error;

	
	/************************************************************************************************/
	// TX Transmitter test using OVERVIEW Measurement Object
	/************************************************************************************************/
	printf("\n\tTX Transmitter - Test using OVERVIEW Measurement Object\n");
	//
	//  This test evaluates the transmitter performances (power frequency error modulation and ACP)
	//
	//  This measurement retreives :
	//
	// 	Power 		-> Transmitted Power from the MS 			4 values : current evaluation , average, min , max
	//  Frequency 	-> Frequency Error							4 values : current evaluation , average, min , max
	//	Modulation	-> Tau Transmit time error					4 values : current evaluation , average, min , max
	//  Modulation 	-> Rho Waveform Quality						4 values : current evaluation , average, min , max
	//  ACP 		-> Lower SideBand Supression at -900 Khz	4 values : current evaluation , average, min , max
	//  ACP 		-> Lower SideBand Supression at +900 Khz	4 values : current evaluation , average, min , max	
	
	// Configure the Overview Measurement Object
	// Setup the CMU level
	CheckCmuError(RSCMU_CDMA2K_ConfOverMCQLev (handle.VALID, 
												-50.0, 							// Set CMU RF Level in dBm
												-14.0, 							// Set Traffic Channel level in dB
												-7.0, 							// Set Pilot Level in dB
												e));
	
	// set the Measurement Condition
	CheckCmuError(RSCMU_CDMA2K_ConfOverCont (handle.VALID, 
												MeasMod,	 					// Use OQPSK for RC 1,2 and HPSK for 3 and 4
												3,								// Set the number of statistic
												RSCMU_REP_SINGLESHOT, 			// Mode Single shot
												RSCMU_STOP3_NONE,				// No Stop On limit
												RSCMU_STEP_MODE_NONE, 			// No Step Mode
												e));
	
	// Trig a measurement Test
	CheckCmuError(RSCMU_CDMA2K_ReadOverview (handle.VALID, 
												MeasMod, 
												&TX_OVERMEAS,e));

											
	// Abort DSP
	CheckCmuError(RSCMU_CDMA2K_AbortOverview (handle.VALID, MeasMod, e));
												

// Display the results
printf("\nTX Overview Measurement\n");
printf("\n\
\tExpected Power\t\t: %6.2lf dBm\n\
\tMeasured Power\t\t: %6.2lf dBm\n\
\tExpected Freq.\t\t: %6.2lf MHz\n\
\tFrequency Error\t\t: %6.2lf Hz\n\
\tTau - Tim.Error\t\t: %6.2lf uS\n\
\tRho - Wave. Qual.\t: %6.2lf \n\
\tLow Side Band sup.\t: %6.2lf dB\n\
\tUpp Side Band sup.\t: %6.2lf dB\n",
TX_OVERMEAS.expectedMSPower.Current,
TX_OVERMEAS.measuredMSPower.Average,
TX_OVERMEAS.carrierFreqError.Current,
TX_OVERMEAS.expectedCarrierFreq.Average,
TX_OVERMEAS.transmitTimeErr.Average,
TX_OVERMEAS.waveformQuality.Average,
TX_OVERMEAS.sidebandSuppressionLower.Average,
TX_OVERMEAS.sidebandSuppressionUpper.Average);


	/************************************************************************************************/
	// TX Transmitter test using POWER Measurement using NPOWER Object
	/************************************************************************************************/
	printf("\nTX Transmitter test - Test POWER using NPOWER Object\n");
	
	// // set measurement condition for Overview Channel quality
	CheckCmuError(RSCMU_CDMA2K_ConfOverCQualContRep (handle.VALID,
												RSCMU_REP_CONTINUOUS,			// Single shot
												RSCMU_STOP3_NONE,				// No stop on limit
												RSCMU_STEP_MODE_NONE, 			// No step mode
												e));

	
	//  Configure the Npower object
	CheckCmuError(RSCMU_CDMA2K_ConfNPowCont (handle.VALID, 
												5, 								// 5 statistics
												RSCMU_REP_SINGLESHOT,			// Single shot
						   						RSCMU_NONE, 					// No Stop
						   						RSCMU_NONE, 					// No Step Mode
						   						e));
	
	
	// start the Channel Quality in continuous mode
	CheckCmuError(RSCMU_CDMA2K_InitOverCQuality (handle.VALID, e));
	
	printf("\tTest MS power - Control bit are set to AUTO mode -> Test MS Close & Open Loop\n");
	
	// Set Power Control BIT in AUTO mode
	CheckCmuError(RSCMU_CDMA2K_ConfOverMCQPCBits (handle.VALID, RSCMU_BIT_AUTO, e));
	
	
	// Loop on power measurement
	for(i=0; i<10; i++)
	{
					   						
		// Trig a power measurement
		CheckCmuError(RSCMU_CDMA2K_ReadScalNPow (handle.VALID, 
												&TX_NPOW.AvgCur,
						   						&TX_NPOW.MinCur, 
						   						&TX_NPOW.MaxCur,
						   						&TX_NPOW.AvgAvg, 
						   						&TX_NPOW.MinAvg,
						   						&TX_NPOW.MaxAvg, 
						   						e));
				  
	
		printf("\t%2d - Power\t\t\t: %6.2lf dBm\n",i+1,TX_NPOW.AvgAvg);
	}
		
		
	printf("\tTest MS power - Control bit are set to HOLD mode -> Test Open Loop \n");
	
	// Set Power Control BIT in HOLD mode
	CheckCmuError(RSCMU_CDMA2K_ConfOverMCQPCBits (handle.VALID, RSCMU_BIT_HOLD, e));
	
	
	// Loop 10 power measurement
	for(i=0; i<10; i++)
	{
					   						
		CheckCmuError(RSCMU_CDMA2K_ReadScalNPow (handle.VALID, 
												&TX_NPOW.AvgCur,
						   						&TX_NPOW.MinCur, 
						   						&TX_NPOW.MaxCur,
						   						&TX_NPOW.AvgAvg, 
						   						&TX_NPOW.MinAvg,
						   						&TX_NPOW.MaxAvg, 
						   						e));
				  
	
		printf("\t%2d - Power\t\t\t: %6.2lf dBm\n",i+1,TX_NPOW.AvgAvg);
	}
	
	
	printf("\tTest power - Control bit are set to ALL UP  -> Test MS MAX Ouput Power\n");
	
	// Set Power Control BIT in ALLUP mode
	CheckCmuError(RSCMU_CDMA2K_ConfOverMCQPCBits (handle.VALID, RSCMU_BIT_AUP, e));
	
	
	// Loop 10 power measurement
	for(i=0; i<10; i++)
	{
					   						
		CheckCmuError(RSCMU_CDMA2K_ReadScalNPow (handle.VALID, 
												&TX_NPOW.AvgCur,
						   						&TX_NPOW.MinCur, 
						   						&TX_NPOW.MaxCur,
						   						&TX_NPOW.AvgAvg, 
						   						&TX_NPOW.MinAvg,
						   						&TX_NPOW.MaxAvg, 
						   						e));
				  
	
		printf("\t%2d - Power\t\t\t: %6.2lf dBm\n",i+1,TX_NPOW.AvgAvg);
	 }
	
		
	// Stop the channel Quality
	CheckCmuError(RSCMU_CDMA2K_AbortOverCQuality (handle.VALID, e));	
	
	// Abort DSP
	CheckCmuError(RSCMU_CDMA2K_AbortNPow (handle.VALID, e));

	/************************************************************************************************/
	// TX Transmitter test using MODULATION Measurement Object
	/************************************************************************************************/
	printf("\nTX Transmitter test - Test MODULATION using MODULATION OVERVIEW Object\n");
	//
	//  This test evaluates the transmitter performances (power frequency error modulation and ACP)
	//
	//  This measurement retreives :
	//

	// Configure the Modulation Object
	CheckCmuError(RSCMU_CDMA2K_ConfModOempLev (handle.VALID, 
												-50.0, 
												-7.4, 
												-7.0,
								 				MeasMod, 
								 				e));
								 			
	CheckCmuError(RSCMU_CDMA2K_ConfModCont (handle.VALID, 
												RSCMU_SCALAR, 
												5,
							  					RSCMU_REP_SINGLESHOT, 
							  					RSCMU_STOP3_NONE,
							  					RSCMU_STEP_MODE_NONE, 
							  					RSCMU_OVERVIEW,
							  					MeasMod, 
							  					e));
							  
	CheckCmuError(RSCMU_CDMA2K_ReadMod (handle.VALID, 
												&TX_MOD_EVM, 
												&TX_MOD_OVER,
						  						RSCMU_OVERVIEW, 
						  						MeasMod, 
						  						e));
						  						
						  						
	// Abort DSP
	CheckCmuError(RSCMU_CDMA2K_AbortMod (handle.VALID, RSCMU_OVERVIEW, MeasMod, e));
	

printf("\n\tTX modulation Overview \n\
\tFrequency Error\t\t: %6.0lf Hz\n\
\tPhase Error\t\t\t: %6.3lf Deg\n\
\tMagnitude Error\t\t: %6.3lf dB\n\
\tEVM RMS\t\t\t\t: %6.3lf %%\n\
\tEVM Peak\t\t\t: %6.3lf %%\n\
\tOrigin Offset\t\t: %6.3lf dB\n\
\tIQ Imbalance\t\t: %6.3lf dB\n\
\tTau\t\t\t\t\t: %6.3lf us\n\
\tRho\t\t\t\t\t: %6.3lf\n",
TX_MOD_OVER.FreqErr.Average,
TX_MOD_OVER.PErrRMS.Average,
TX_MOD_OVER.MErrRMS.Average,
TX_MOD_OVER.EVMRMS.Average,
TX_MOD_OVER.EVMPeak.Average,
TX_MOD_OVER.CarrierFeedthrough.Average,
TX_MOD_OVER.IQImbalance.Average,
TX_MOD_OVER.TransmitTimeError.Average,
TX_MOD_OVER.WaveformQuality.Average); 	

	
	/************************************************************************************************/
	// Change Base Station RF level during a call
	/************************************************************************************************/
	printf("\n\tChange Base Station RF level during a call\n");
	CheckCmuError(RSCMU_CDMA2K_ProcBSSRfPowCDMA (handle.VALID, -85.0, e));
	
	

	/************************************************************************************************/
	// TX Transmitter - test MAX Output power
	/************************************************************************************************/
	printf("\nTX Transmitter test - Test MAX Output power\n");
	//
	// This test the maximum ouput power that the phone can transmit
	//
	// Measurement object retreives :
	// Power 				-> Maximum Ouput Power 			: 4 values { Current evalutaion, Average , Min , Max)
	// Wave form quality 	-> Rho 							: 4 values { Current evaluation , Average , Min, Max)
	
	// Set the measurement condition
	CheckCmuError(RSCMU_CDMA2K_ConfPowMaxOutCont (handle.VALID, 
												3,								// Configure the number of statistics
												RSCMU_REP_SINGLESHOT,			// Configure Single shot mode
												RSCMU_STOP3_NONE,				// Configure No Stop condition
												RSCMU_STEP_MODE_NONE, 			// Configure No Step mode
												MeasMod, 						// Configure OQPSK or HPSK mode
												e));

	// set the FW link 
	CheckCmuError(RSCMU_CDMA2K_ConfPowMaxOutLev (handle.VALID, 
												-94.0, 							// Configure CMU RF ouput level
												-7.4, 							// Configure the TCH level in dB
												-7.0,							// Configure the pilot level in dB
												MeasMod, 						// Configure OQPSK or HPSK mode
												e));
	
	// Trig a measurement
	CheckCmuError(RSCMU_CDMA2K_ReadScalPowMaxOut (handle.VALID, 
												&TX_MMPOW.PowCurr,				// Get Max Output Power from the last evaluation
												&TX_MMPOW.PowAvg, 				// Get Max Output Power average values over the n evaluations
												&TX_MMPOW.PowMin,				// Get Max Output Power min values over the n evaluations
												&TX_MMPOW.PowMax, 				// Get Max Output Power max values over the n evaluations
												&TX_MMPOW.RhoCurr,				// Get WaveForm Quality from the last evaluation             
												&TX_MMPOW.RhoAvg, 				// Get WaveForm Quality average values over the n evaluations
												&TX_MMPOW.RhoMin,				// Get WaveForm Quality min values over the n evaluations    
												&TX_MMPOW.RhoMax, 				// Get WaveForm Quality max values over the n evaluations    
												&TX_MMPOW.MOT,					// Get Measurements Out of Tolerance
												MeasMod, 						// Set mode to OQPSK or HPSK
												e));
												
													
	// Abort the DSP
	CheckCmuError(RSCMU_CDMA2K_AbortPowMaxOut (handle.VALID, MeasMod, e));
	
// Display results
printf("\nTX MAX output power\n\
\tAverage value\t\t: %6.2lf dBm\n\
\tMaximum value\t\t: %6.2lf dBm\n\
\tMinimum value\t\t: %6.2lf dBm\n\
\nTX Waveform Quality\n\
\tAverage Rho\t\t\t: %6.3lf\n\
\tMaximum Rho\t\t\t: %6.3lf\n\
\tMinimum Rho\t\t\t: %6.3lf\n",
TX_MMPOW.PowAvg,
TX_MMPOW.PowMin,
TX_MMPOW.PowMax,
TX_MMPOW.RhoAvg,
TX_MMPOW.RhoMax,
TX_MMPOW.RhoMin); 	
	
	
	/************************************************************************************************/
	// RX Receivertest - Sensitivity test
	/************************************************************************************************/
	printf("\nRX Receiver test - Test sensitivity\n"); 	
	//
	// This test receiver sensitivity of the phone at a base station level
	//
	// Measurement object retreives :
	//
	//  FER Frame Error Ratio
	//  Number of Frames in Error
	//  Number of Frames Transmitted
	//	Confidence level
	//
	
	// Configure he RECEVER QUALITY Object
	// Select the TSET
	CheckCmuError(RSCMU_CDMA2K_ConfRQFerFSSCTset (handle.VALID, 
												RSCMU_CDMA_T1, 					// Select the Tset measurement
												e));
	// Configure the number of frames
	CheckCmuError(RSCMU_CDMA2K_ConfRQFerFCHContTsetFram (handle.VALID, 
												RSCMU_CDMA_T1,
												1000, 							// Configure the number of frames for the test
												e));
	// Configure the test condition
	CheckCmuError(RSCMU_CDMA2K_ConfRQFerFCHContTsetRep (handle.VALID,
												RSCMU_CDMA_T1,
									  			RSCMU_REP_SINGLESHOT,			// Configure measurement to Single Shot
									  			RSCMU_STOP2_ALEX,				// Configure measurement to Stop on All limit exceeded
									  			RSCMU_STEP_MODE_NONE, 			// Configure No Step mode
									  			e));
									  			
	// Set FER limit
	CheckCmuError(RSCMU_CDMA2K_ConfRQFerFCHLimTsetMFer (handle.VALID, 
												RSCMU_CDMA_T1,
												3.0, 							// Configure FER Limit
												RSCMU_ON,
												e));   
	// Configure the TCH and SCH levels
	CheckCmuError(RSCMU_CDMA2K_ConfRQFerFSSCContTsetLev (handle.VALID, 
												RSCMU_CDMA_T1,
												-7.0, 							// Configure FW Pilot Level
												-15.6,							// Configure FW Fundamental Traffic channel level
												-13.5, 							// Configure FW Supplementary channel level
												e));
	
	
	// Configure the Base Station total RF output level in dBm
	CheckCmuError(RSCMU_CDMA2K_ConfRQFerFSSCTsetPowCDMA (handle.VALID, 
												RSCMU_CDMA_T1,
												-104.0, 						// Base CMU Station RF level
												e));

	// Start a FER test
	CheckCmuError(RSCMU_CDMA2K_InitRQFerFCH (handle.VALID, e));
	Delay(0.15); 
	
	state		= 0;	
	meas_status = 0;
	index       = 0;
	do
	{
		
		// Query call status
		RSCMU_CDMA2K_ProcSigState (handle.VALID, &state, e);
		if (state != RSCMU_CDMA_STATE_CEST)
		{
			strcpy(e,"Call drop during FER measurement\0");
			goto Error;
		}
		
		// Query Measurement Status
		CheckCmuError(RSCMU_CDMA2K_FetchRQFerFCHStats (handle.VALID, 
												&meas_status,					// Query the meas. status
									 			&counterStat, 
									 			&counterEval, 
									 			e));
	
	} while (meas_status != RSCMU_MEAS_STAT_RDY);
	
	// Query FER measurement  result
	CheckCmuError(RSCMU_CDMA2K_FetchRQFerFCH (handle.VALID, 
												&RX_FER.FER, 					// Get FER
												&RX_FER.Errors,					// Get Number of Frames in error
												&RX_FER.FramesReceived, 		// Get Number of frames received
												&RX_FER.CL, 					// Get Confidence level
												e));

	// Stop the DSP application
	CheckCmuError(RSCMU_CDMA2K_AbortRQFerFCH (handle.VALID, e));

printf("\n\tRX test - Receiver Quality sensitivity\n\
\tFrame Error Rate\t: %6.3lf %%\n\
\tNumber of Errors\t: %6d\n\
\tFrames received\t\t: %6d\n\
\tConfidence Level\t: %6.2lf %%\n",
RX_FER.FER,			
RX_FER.Errors, 		
RX_FER.FramesReceived,	
RX_FER.CL);	

	/************************************************************************************************/
	// Change BASE Station RF level
	/************************************************************************************************/
	CheckCmuError(RSCMU_CDMA2K_ProcBSSRfPowCDMA (handle.VALID, -50.0, e));
	Delay(3.0);
	

	/************************************************************************************************/
	// TX Transmitter - test Min Output power
	/************************************************************************************************/
	printf("\nTX Transmitter test - Test MIN Output power\n");
	//
	// This test the minimum ouput power that the phone can transmit
	//
	// Measurement object retreives :
	// Power 				-> Minimum Ouput Power 			: 4 values { Current evalutaion, Average , Min , Max)
	// Wave form quality 	-> Rho 							: 4 values { Current evaluation , Average , Min, Max)
	
	// Set the measurement condition
	CheckCmuError(RSCMU_CDMA2K_ConfPowMinOutCont (handle.VALID, 
												3,								// Configure the number of statistics
												RSCMU_REP_SINGLESHOT,			// Configure Single shot mode
												RSCMU_STOP3_NONE,				// Configure No Stop condition
												RSCMU_STEP_MODE_NONE, 			// Configure No Step mode
												MeasMod,	 					// Configure OQPSK or HPSK mode
												e));

	// set the FW link 
	CheckCmuError(RSCMU_CDMA2K_ConfPowMinOutLev (handle.VALID, 
												-25.0, 							// Configure CMU RF ouput level
												-7.4, 							// Configure the TCH level in dB
												-7.0,							// Configure the pilot level in dB
												MeasMod,	 					// Configure OQPSK or HPSK mode
												e));
	
	// Trig a measurement
	CheckCmuError(RSCMU_CDMA2K_ReadScalPowMinOut (handle.VALID, 
												&TX_MMPOW.PowCurr,				// Get Min Output Power from the last evaluation
												&TX_MMPOW.PowAvg, 				// Get Min Output Power average values over the n evaluations
												&TX_MMPOW.PowMin,				// Get Min Output Power min values over the n evaluations
												&TX_MMPOW.PowMax, 				// Get Min Output Power max values over the n evaluations
												&TX_MMPOW.RhoCurr,				// Get WaveForm Quality from the last evaluation             
												&TX_MMPOW.RhoAvg, 				// Get WaveForm Quality average values over the n evaluations
												&TX_MMPOW.RhoMin,				// Get WaveForm Quality min values over the n evaluations    
												&TX_MMPOW.RhoMax, 				// Get WaveForm Quality max values over the n evaluations    
												&TX_MMPOW.MOT,					// Get Measurements Out of Tolerance
												MeasMod,	 					// Set mode to OQPSK or HPSK
												e));
												
													
	// Abort the DSP
	CheckCmuError(RSCMU_CDMA2K_AbortPowMinOut (handle.VALID, MeasMod, e));
	
// Display results
printf("\n\tTX MIN output power\n\
\tAverage value\t\t: %6.2lf dBm\n\
\tMaximum value\t\t: %6.2lf dBm\n\
\tMinimum value\t\t: %6.2lf dBm\n",
TX_MMPOW.PowAvg,
TX_MMPOW.PowMin,
TX_MMPOW.PowMax);

	
	// Test GATED power for RC1 and RC2
	if ((BST.RC_RFCH == 1) || (BST.RC_RFCH == 2))
	{
		
		/************************************************************************************************/
		// Change RATE to 1/8
		/************************************************************************************************/
		printf("\nSet RATE to 1/8");
		CheckCmuError(RSCMU_CDMA2K_ProcSConfSclLoopFCHFr (handle.VALID, 
												RSCMU_CDMA_EIGHTH,
												BST.ServOption, 
												e));
		Delay(1.5);	

		/************************************************************************************************/
		// TX Transmitter test using GATED Measurement Object
		/************************************************************************************************/
		printf("\nTX Transmitter - Test GATED Output power\n");	

		CheckCmuError(RSCMU_CDMA2K_ConfPowGatedOutCont (handle.VALID, 
												RSCMU_SCALAR, 
												5,
											  	RSCMU_REP_SINGLESHOT,
											  	RSCMU_STOP3_NONE,
											  	RSCMU_STEP_MODE_NONE, 
											  	e));
		//									  	
		CheckCmuError(RSCMU_CDMA2K_ConfPowGatedOutContPCG (handle.VALID, 64, e));

		//
		CheckCmuError(RSCMU_CDMA2K_ReadScalPowGatedOut (handle.VALID, 
												&TX_GATED.Power, 
												&TX_GATED.LimMATC,
												&TX_GATED.MOT, 
												e));

		// ABOART DSP
		CheckCmuError(RSCMU_CDMA2K_AbortPowGatedOut (handle.VALID, e));
	
printf("\n\tTX GATED output power\n\tTemplate\t\t: %s \n", MATCH[TX_GATED.LimMATC]);

	
		/************************************************************************************************/
		// Change RATE to FULL
		/************************************************************************************************/
		CheckCmuError(RSCMU_CDMA2K_ProcSConfSclLoopFCHFr (handle.VALID, 
												RSCMU_CDMA_FULL,
												BST.ServOption, 
												e));
	
		Delay(1.5);	
	
	}

	
	// Test CODE DOMAIN POWER
	if ((BST.RC_RFCH == 3) || (BST.RC_RFCH == 4))
	{
		/************************************************************************************************/
		// TX Transmitter test using GATED Measurement Object
		/************************************************************************************************/
		printf("\nTX Transmitter - Test CODE DOMAIN POWER\n");	
	

		// TX CDP
		// Bug CheckCmuError(RSCMU_CDMA2K_ConfCDPowCpcCLev (handle.VALID, -50.0, -7.4, -7.0, e));
	
		// Configure the Peak CDP for the measurement										
		CheckCmuError(RSCMU_CDMA2K_ConfCDPowPCDEPCont (handle.VALID, 
												RSCMU_ARRAY, 
												10,
												RSCMU_REP_SINGLESHOT,
												RSCMU_STOP3_NONE,
												RSCMU_STEP_MODE_NONE, 
												e));
	
		// Trig a Measurement
		CheckCmuError(RSCMU_CDMA2K_ReadScalCDPowPCDEP (handle.VALID, 
												&TX_CDP, 
												e));
					  


printf("\n\tTX Code Domain Power\n");
printf("\n\
\tMS Power\t\t\t: %6.2lf dBm\n\
\tCarrier Feedthro.\t: %6.2lf dB\n\
\tFreq. Error\t\t\t: %6.2lf Hz\n\
\tRho (Wave. Qual)\t: %6.3lf\n",
TX_CDP.msPower.Average,
TX_CDP.CarrierFeedthrough.Average,
TX_CDP.FrequencyError.Average,
TX_CDP.rho.Average);



		// I vector match Limit ???
		CheckCmuError(RSCMU_Query(handle.VALID,"CALC:ARR:CDP:PCDEP:ISIG:AVER:MATC:LIM?",buffin,RSCMU_OFF,e));
		sscanf(buffin,"%d",&meas_status);


printf("\n\tI Template\t\t\t: %s \n", MATCH[meas_status]);	


		CheckCmuError(RSCMU_Query(handle.VALID,"CALC:ARR:CDP:PCDEP:QSIG:AVER:MATC:LIM?",buffin,RSCMU_OFF,e));
		sscanf(buffin,"%d",&meas_status);

printf("\tQ Template\t\t\t: %s \n", MATCH[meas_status]);	
		
		
		// Abort DSP
		CheckCmuError(RSCMU_CDMA2K_AbortCDPowPCDEP (handle.VALID, e));
	
	
	
	}
	
	/************************************************************************************************/
	// release the call from the CMU
	/************************************************************************************************/
	CheckCmuError(RSCMU_CDMA2K_ReleaseCallfromBTS (handle.VALID, 2, e));

	Delay(3.0);
	
	// Set RF channel
	CheckCmuError(RSCMU_CDMA2K_ConfBSSRfChannel (handle.VALID, BST.RFChan , e));

	

} while (TestLoop == 1);

	if (handle.CDMA1900NS != 0) RSCMU_CDMA2K_Close (handle.CDMA1900NS, e);
	if (handle.CDMA1900S != 0) RSCMU_CDMA2K_Close (handle.CDMA1900S, e);
	if (handle.CDMA800NS != 0) RSCMU_CDMA2K_Close (handle.CDMA800NS, e);
	if (handle.CDMA800S != 0) RSCMU_CDMA2K_Close (handle.CDMA800S, e);
	if (handle.BASE !=0) RSCMU_Close_Base (handle.BASE, e);
	
	
	return 0;

Error:	

	
	MessagePopup ("Error", e);
	
	// Close trace
	if (handle.BASE !=0) RSCMU_Trace_OFF (handle.BASE, e);	
	
	if (handle.CDMA1900NS != 0) RSCMU_CDMA2K_Close (handle.CDMA1900NS, e);
	if (handle.CDMA1900S != 0) RSCMU_CDMA2K_Close (handle.CDMA1900S, e);
	if (handle.CDMA800NS != 0) RSCMU_CDMA2K_Close (handle.CDMA800NS, e);
	if (handle.CDMA800S != 0) RSCMU_CDMA2K_Close (handle.CDMA800S, e);
	if (handle.BASE !=0) RSCMU_Close_Base (handle.BASE, e);
	
	
	return -1;
}


/************************************************************************************************/
// Select RF path Input Output
/************************************************************************************************/
int RF_Path(ViSession	handle, int	Netw, int RFIO, double RFATT) {

	ViStatus 	cmu_status;
	ViChar 		errmsg[1024];

	switch (Netw) {
#ifdef _GSM	
		case GSM:
			cmu_status = RSCMU_GSM_Sig_Inp (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_GSM_Sig_Outp (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;
				
			cmu_status = RSCMU_GSM_Sig_SourCorrLossOutp (handle, RFIO,RFATT, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_GSM_Sig_SensCorrLossInp (handle, RFIO, RFATT,errmsg);
			if (cmu_status != 0) goto error;
			
			
		break;
#endif
#ifdef _TDMA		
		case TDMA:
		
			cmu_status = RSCMU_TDMA_InpState (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_TDMA_OutpState (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_TDMA_SenseCorrLossInp (handle, RFIO, RFATT, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_TDMA_SenseCorrLossOutp (handle, RFIO, RFATT, errmsg);
			if (cmu_status != 0) goto error;			
		
		break;
#endif
#ifdef _AMPS		
		case AMPS:
			cmu_status = RSCMU_AMPS_InpState (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_AMPS_OutpState (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_AMPS_SenseCorrLossInp (handle, RFIO, RFATT, errmsg);
			if (cmu_status != 0) goto error;
			
			cmu_status = RSCMU_AMPS_SenseCorrLossOutp (handle, RFIO, RFATT, errmsg);
			if (cmu_status != 0) goto error;			
				
		
		break;
#endif		
		case CDMA:
			
	
 			// RF Input		   	
		   	cmu_status = RSCMU_CDMA2K_InpState (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;			
			
 			// RF Compensation   										
			cmu_status = RSCMU_CDMA2K_SenseCorrLossInp (handle, RFIO, RFATT, errmsg);
			if (cmu_status != 0) goto error;			
			
   			// RF Output 2							
		   	cmu_status = RSCMU_CDMA2K_OutpState (handle, RFIO, errmsg);
			if (cmu_status != 0) goto error;			
			
			cmu_status = RSCMU_CDMA2K_SourceCorrLossOutp (handle, RFIO, RFATT,	errmsg);		
			if (cmu_status != 0) goto error;			
			
		break;
	
	}

 return 0;

error:

	MessagePopup("Error", errmsg);
 return cmu_status;
	
}


/************************************************************************************************/
/************************************************************************************************/
int Setup_CDMA_BST( ViSession 	handle, tBST	BST) 
{
	
	// Declaration

	ViChar               	cmu_errmsg[1024];          					// Error Message from CMU driver	
	int						_error;
	
	
	// SET RF 
	// Turn OFF the signaling
	CheckCmuError(RSCMU_CDMA2K_ProcSigAction (handle, 
											RSCMU_SOFF, 				// Signaling OFF
											cmu_errmsg));
    
    
   	// Setup band
   	CheckCmuError(RSCMU_CDMA2K_ConfNstandard (handle, 
   											BST.Netw, 					// band 
   											cmu_errmsg));

   	// Conf Base Station
   	// Set RF channel
	 CheckCmuError(RSCMU_CDMA2K_ConfBSSRfChannel (handle, 
   											BST.RFChan,					// RF Channel
   											cmu_errmsg));
				   
   
	CheckCmuError(RSCMU_CDMA2K_ConfBSSRfPowCDMA (handle, 
											BST.PowerLev, 				// Total ouput power
											cmu_errmsg));
											
	CheckCmuError(RSCMU_CDMA2K_ConfBSSFSyncLevel (handle, 
											BST.SyncLev, 				// Synchro Channel Level
											RSCMU_ON, 
											cmu_errmsg));
											
	CheckCmuError(RSCMU_CDMA2K_ConfBSSFPilotChLevel (handle, 
											BST.PilotLev, 				// Pilot Channel Level
											RSCMU_ON, 
											cmu_errmsg));
											
	CheckCmuError(RSCMU_CDMA2K_ConfBSSFFCHLevel (handle, 
											BST.TrafficLev, 			// Traffic Channel Level
											RSCMU_ON, 
											cmu_errmsg));
											
	CheckCmuError(RSCMU_CDMA2K_ConfBSSFFCHChannel (handle, 
											BST.TrafficChan, 			// Traffic Channel
											cmu_errmsg));
											
	CheckCmuError(RSCMU_CDMA2K_ConfBSSFPCHLevel (handle, 				// Paging Channel Level
											BST.PagingLev,
											RSCMU_ON, cmu_errmsg));
	
	CheckCmuError(RSCMU_CDMA2K_ConfBSSFFCHFrameOffset (handle, 0, cmu_errmsg)); // Frame Offset
	

	// Set NETWORK
	// Set SID
	CheckCmuError(RSCMU_CDMA2K_ConfNetSysSID (handle, 
											BST.NETW_SID, 				// SID
											cmu_errmsg));
	

	// Set NID									
	CheckCmuError(RSCMU_CDMA2K_NetIdenNID (handle, 
											BST.NETW_NID, 				// NID
											cmu_errmsg));
	
	// Set protocol revision
	CheckCmuError(RSCMU_CDMA2K_ConfNetSysPRevision (handle, 
											BST.NET_PREV,
											cmu_errmsg));
	
	

	return(0);
	
Error:   	
	return(-1);
	
}


/************************************************************************************************/
/************************************************************************************************/
int Setup_CDMA_Service(ViSession 	handle, tBST	BST)
{

	// Declaration

	ViChar               	cmu_errmsg[1024];          					// Error Message from CMU driver	
	int						_error;

	
	// SET SERVICE
	// Set service Class
	CheckCmuError(RSCMU_CDMA2K_ConfSConfSclService (handle, 
											BST.ServClass,
											cmu_errmsg));

	// Configure Test loopback
	if (BST.ServClass == RSCMU_SERVICE_LOOP)
	{
		// Set Service option
		CheckCmuError(RSCMU_CDMA2K_ConfSConfSclLoopSOption (handle,
											BST.ServOption, 
											cmu_errmsg));
	
		// Setup  Radio Configuration
		CheckCmuError(RSCMU_CDMA2K_ConfSConfSclLoopFCHRC (handle, 
											BST.RC_FFCH, 
											BST.RC_RFCH,
											BST.ServOption, 
											cmu_errmsg));
	
		// Setup RATE
		CheckCmuError(RSCMU_CDMA2K_ConfSConfSclLoopFCHFr (handle, 
											BST.RATE,
											BST.ServOption, 
											cmu_errmsg));
	}
	
	
	// Setup Service class in speech mode
	if (BST.ServClass == RSCMU_SERVICE_SPEECH)
	{

		CheckCmuError(RSCMU_CDMA2K_ConfSConfSclSpeechSOpt (handle,
											BST.ServOption, 
											cmu_errmsg));
	
		// Setup  Radio Configuration
		CheckCmuError(RSCMU_CDMA2K_ConfSConfSclSpeechFCHRC (handle, 
											BST.RC_FFCH, 
											BST.RC_RFCH,
											BST.ServOption, 
											cmu_errmsg));
	

	}
	
	// Setup Service class in data service
	
	return(0);
	
Error:   	
	return(-1);
	
}
